class Danila_rama_classify_params:
    def __init__(self, rama_classify_version, rama_classify_model):
        self.rama_classify_version = rama_classify_version
        self.rama_classify_model = rama_classify_model