class Danila_rama_text_detect_params:
    def __init__(self, rama_detect_version, rama_text_detect_version):
        self.rama_detect_version = rama_detect_version
        self.rama_text_detect_version = rama_text_detect_version