from data.result import Rama_prod, balka_prod

RAMA_PRODS = {Rama_prod.Rama_Prod.altai: '1923',
              Rama_prod.Rama_Prod.balakovo:'1487',
              Rama_prod.Rama_Prod.begickaya:'12',
              Rama_prod.Rama_Prod.promlit:'33',
              Rama_prod.Rama_Prod.ruzhimmash:'1275',
              Rama_prod.Rama_Prod.tihvin:'1378',
              Rama_prod.Rama_Prod.uralvagon:'5'}

BALKA_PRODS = {balka_prod.Balka_Prod.altai: '1923',
              balka_prod.Balka_Prod.begickaya:'12',
              balka_prod.Balka_Prod.promlit:'33',
              balka_prod.Balka_Prod.ruzhimmash:'1275',
              balka_prod.Balka_Prod.tihvin:'1378'}